#version 410 core

in vec3 worldNormal;
in vec2 texCoord;
in vec3 worldPos;
in vec4 posLightSpace;
in mat3 TBN;

// Switch to enable/disable Normal Mapping and use interpolated normals instead
uniform int normalMapping;

// Texture Samples
uniform sampler2D color_texture;
uniform sampler2D normalMap;

// ShadowMap Sampler
uniform sampler2DShadow shadowMap;

uniform vec3 cameraPosition;
uniform mat4 model;


//Clustered Light stuff
struct DirectionalLight
{
   vec3 lightColor;
   vec3 lightPos;
   float ambientIntensity;
};

struct Material
{
   float specularIntensity;
   float specularPower;
   float transparency;
}; 

uniform DirectionalLight sunLight; 
uniform Material mat;

// Final fragment Color
layout (location = 0) out vec4 fragColor;


// Tests if fragment is in shadow for given Fragposition
float shadowValue(vec4 fragLightSpace) {
	
	// This is the test actually. TextureProj performs a lookup in a given Sampler with a given TextureCoordinate
	// and returns the texel.
	// Since our Sampler is a shadow Sampler and the underlying texture has comparison enabled
	// textureProj returns the result of a depth Test (so either 0.0 OR 1.0)
	float shadowMult = textureProj(shadowMap, fragLightSpace);
	return shadowMult;
}

void main() {

	vec4 surfaceColor = texture(color_texture, texCoord);
	// Vectors needed for lighting
	vec3 surfaceToLight = normalize(sunLight.lightPos - worldPos);
    vec3 surfaceToCamera = normalize(cameraPosition - worldPos);
	vec3 normal = normalize(worldNormal);

	if(normalMapping == 1){
		// Retrieve normal from texel
        normal = texture(normalMap, texCoord).rgb;
        normal = normalize(normal * 2.0 - 1.0);
		// Transform to TBN space to make orientation independent
		normal = normalize(TBN * normal);   
	}


	//ambient
    vec3 ambient = sunLight.ambientIntensity * surfaceColor.rgb * sunLight.lightColor;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, normalize(sunLight.lightPos)));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * sunLight.lightColor;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(surfaceToLight, normal))), mat.specularPower);
    vec3 specular = mat.specularIntensity * specularCoefficient * sunLight.lightColor;

	// Put everything(!) together here.
	// There is always ambient lighting
	// Multiplay diffuse & specular part with 0 if Fragment is in Shadow.
	fragColor = vec4(ambient + (1.0 - shadowValue(posLightSpace)) * ( diffuse + specular), mat.transparency);
	
}